use serde::{Deserialize, Serialize};
use sqlx::error::Error;
use sqlx::types::Uuid as UuidType;
use sqlx::{Executor, Postgres};
use uuid::Uuid;

#[derive(Deserialize)]
pub struct PartialTrackedPlaylist {
    pub playlist_id: Option<Uuid>,
    pub playlist_size: i32,
    pub tracking_user: Option<String>,
    pub tracking_type: Option<String>,
}

impl PartialTrackedPlaylist {
    pub async fn record(
        &self,
        pool: impl Executor<'_, Database = Postgres> + Copy,
    ) -> Result<TrackedPlaylist, Error> {
        let uuid = Uuid::new_v4();

        sqlx::query!(
            r#"
            INSERT INTO "tracked_playlist"
            (id, playlist_id, playlist_size, tracking_user, tracking_type)
            VALUES ($1, $2, $3, $4, $5)
            "#,
            uuid,
            self.playlist_id,
            self.playlist_size,
            self.tracking_user,
            self.tracking_type,
        )
        .execute(pool)
        .await?;

        TrackedPlaylist::rule(pool, uuid).await
    }
}

#[derive(Serialize)]
pub struct TrackedPlaylist {
    pub id: Uuid,
    pub playlist_id: Option<UuidType>,
    pub playlist_size: i32,
    pub tracking_user: String,
    pub tracking_type: String,
}

impl TrackedPlaylist {
    pub async fn all(pool: impl Executor<'_, Database = Postgres>) -> Result<Vec<Self>, Error> {
        sqlx::query_as!(Self, r#"SELECT * FROM "tracked_playlist""#)
            .fetch_all(pool)
            .await
    }

    pub async fn rule(
        pool: impl Executor<'_, Database = Postgres>,
        uuid: Uuid,
    ) -> Result<Self, Error> {
        sqlx::query_as!(
            Self,
            r#"SELECT * FROM "tracked_playlist" WHERE id = $1"#,
            uuid
        )
        .fetch_one(pool)
        .await
    }
}
